//
//  Poisson.swift
//  Classe - Aquarium
//
//  Created by Alain on 2014-10-12.
//  Copyright (c) 2014 Production sur support. All rights reserved.
//
//  Created by Alain on 2014-10-11.
//  Copyright (c) 2014 Production sur support. All rights reserved.
//
//  ============================================================================================
//  À l'usage exclusif des étudiants et étudiantes de
//  Techniques d'Intégration Multimédia
//  du cégep de Saint-Jérôme.
//  --------------------------------------------------------------------------------------------
//  Il est interdit de reproduire, en tout ou en partie, à des fins commerciales,
//  le code source, les scènes, les éléments graphiques, les classes et
//  tout autre contenu du présent projet sans l’autorisation écrite de l'auteur.
//
//  Pour obtenir l’autorisation de reproduire ou d’utiliser, en tout ou en partie,
//  le présent projet, veuillez communiquer avec:
//
//  Alain Boudreault, aboudrea@cstj.qc.ca
//
//  ============================================================================================
//  Étape 3 - Ajouter une interface visuelle à la classe Poisson
//

import Foundation

// Étape 3.1 - Importer le frameWork UIKit
import UIkit

// Étape 1 - déclarer le protocole
protocol PoissonDelegate {
    // étape 2 - déclarer les méthodes du protocole
    func poissonAFaim(sender:Poisson)
}

// Étape 3.2 - classe Poisson étend UIImageView
class Poisson:UIImageView {

    // Propriétés
    var nomPoisson:String
    var nomImage:String
    var force:Int
    let seuilDangerDeMort = 10
    // Étape 3 - Définir une propriété - delegate - de type 'NomDuProtocole'
    /// Propriété pour stocker le délégué éventuel de l'instance Poisson
    var delegate:PoissonDelegate?
    
    // Constructeur
    /**
      Construire un poisson à partir d'un nom et d'une force
    */
    init(nomPoisson:String, force:Int) {
       println("Je suis le constructeur de Poisson.")
       self.nomPoisson  = nomPoisson
       self.force       = force
       self.nomImage    = "n/a"
        
       // Étape 3.2 - Appel de super.init()
       // Note:  Il faut renseigner les propriétés locales
       // avant l'appel super.init()
       // Il faut utiliser le constructeur de la super classe
       // Il n'est pas possible d'utiliser un init de convenance de la super classe.
       // Le contructeur de UIImageView est:  init(image: UIImage!)
       // Par contre, il ne permet pas de préciser une taille pour l'image.
       // Nous allons donc faire appel au contructeur de la super classe de UIImageView C-A-D, UIView(frame: CGRect)
       //
       
       // Créer une View à la position (20,150) de dimension (100, 50)
       super.init(frame: CGRectMake(20, 150, 100, 50))
       image = UIImage(named: "p13-frame-1.gif")

        // Créer l'animation à partir d'une suite d'images
        var images:[UIImage] = []
        // Note: Il y a 8 images pour l'animation du poisson P13
        for i in 1...8 {
            images.append(UIImage(named:"p13-frame-\(i).gif"))
        }
        
        animationImages = images
        animationDuration = 1.5
        startAnimating()
        
    } // init()

    
    // Constructeur
    /**
    Construire un poisson animé à x,y hauteur et largeur et délégué
    */
    init(nomPoisson:String, nomImage:String, nbImages:Int, posX:CGFloat, posY:CGFloat, largeur:CGFloat, hauteur:CGFloat, leDelegate:PoissonDelegate) {
        println("Je suis le constructeur de \(nomPoisson).")
        self.nomPoisson  = nomPoisson
        self.force       = 20
        self.nomImage    = nomImage
        
        delegate         = leDelegate
        
        // Note:  Il faut renseigner les propriétés locales
        // avant l'appel super.init()
        super.init(frame: CGRectMake(posX , posY, largeur, hauteur))

        // Image sans anime - poisson mort
        image = UIImage(named: "mort.png")

        // Créer l'animation à partir d'une suite d'images
        var images:[UIImage] = []
        for i in 1...nbImages {
            images.append(UIImage(named: "\(nomImage)-frame-\(i).gif"))
        }
        
        animationImages = images
        animationDuration = 1.5
        startAnimating()

        
    } // init()
    
    
    /**
      Cette étrange méthode est obligatoire lorsque la super classe est de type 'Fondation'
    */
    required init(coder aDecoder: NSCoder) {
        fatalError("init(coder:) has not been implemented")
    }


    // Constructeur de convenance
    /**
    Construire un poisson pas de nom et de force zéro!
    Note: Nous avons changé la signature car la super classe possède déjà un init()
    */
    convenience init(rien:Int = 0){
        self.init(nomPoisson:"Pas de nom", force:0)
    } // convenience init()
    
    
    // Destructeur
    /**
      Méthode éxécuté à la libération de l'intance
    */
    deinit{
        println("Je suis le fossoyeur de \(nomPoisson)")
    }
    
    // Méthodes

    /**
      Méthode pour épuiser artificiellement le poisson
    */
    func epuiserLePoisson(force:Int){
        
        self.force -= force
    
        if self.force < 10 {
            println("Le poisson \(nomPoisson) est très très faible")
            // Étape 4 - gérer le protocole
            delegate?.poissonAFaim(self)
        }
    } // epuiserLePoisson

    /**
    Méthode pour nourrir le poisson
    */
    func nourrirLePoisson(nourriture:Int){
        force += 2 * nourriture
    } // nourrirLePoisson

    /**
      Je retourne une jolie description du poisson
    */
    func quiSuisJe() ->String {
        return "\n------------------\nJe suis le poisson \(nomPoisson) et j'ai une force de \(force)\n"
    } // quiSuisJe()
    

    
} // class Poisson